/* demo_xy_problems_4.c
 * 
 * Copyright (C) 2024 Marcin Kulbaka
 * 
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 3 of the License, or (at
 * your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
 */
 
 
  /*
 * problems 31 .. 40
 */

#include <stdio.h>

#include "lbmn_mesh_xy_D.h"
#include "demo_xy_problems_4.h"



int
lbmn_demo_xy_problem_31_2_plates_401_401_casing(
			unsigned char* ptr_is_pre_defined,/*output*/
			unsigned int* ptr_size_row,
            unsigned int* ptr_size_col,
            double* ptr_x_start,
            double* ptr_x_stop,
            double* ptr_y_start,
            double* ptr_y_stop,
			
			double** ptrptr_V,
			unsigned int** ptrptr_Bnd,
			double** ptrptr_E_x,
			double** ptrptr_E_y,
			double** ptrptr_E_1,
			double** ptrptr_E_2,
            unsigned int** ptrptr_isolines_V,
            unsigned int** ptrptr_isolines_E_x,
            unsigned int** ptrptr_isolines_E_y,
            unsigned int** ptrptr_isolines_E_1,
            unsigned int** ptrptr_isolines_E_2,
			unsigned char** ptrptr_workbench_colours_r,
			unsigned char** ptrptr_workbench_colours_g,
			unsigned char** ptrptr_workbench_colours_b,
			unsigned char** ptrptr_workbench_bw_colours_r,
			unsigned char** ptrptr_workbench_bw_colours_g,
			unsigned char** ptrptr_workbench_bw_colours_b)
{
    /* daclaration of variables */

	unsigned int problem_size_row = 401;
    unsigned int problem_size_col = 401;
    double problem_x_start = -200.0;
    double problem_x_stop  = +200.0;
    double problem_y_start = -200.0;
    double problem_y_stop  = +200.0;

    double       voltage        = 0.0;
    unsigned int boundary       = 0;
    unsigned int i_row_start    = 0;
    unsigned int i_row_stop     = 0;
    unsigned int i_col_start    = 0;
    unsigned int i_col_stop     = 0;


    int ret_val = 0;

    /* instructions */

	(*ptr_is_pre_defined) = 0;/* problem to be solved numerically */

    ret_val = lbmn_mesh_xy_problem_allocate(
        	    ptr_size_row,/*output*/
                ptr_size_col,
                ptr_x_start,
                ptr_x_stop,
                ptr_y_start,
                ptr_y_stop,
			    ptrptr_V,
			    ptrptr_Bnd,
			    ptrptr_E_x,
			    ptrptr_E_y,
			    ptrptr_E_1,
			    ptrptr_E_2,
                ptrptr_isolines_V,
                ptrptr_isolines_E_x,
                ptrptr_isolines_E_y,
                ptrptr_isolines_E_1,
                ptrptr_isolines_E_2,
			    ptrptr_workbench_colours_r,
			    ptrptr_workbench_colours_g,
			    ptrptr_workbench_colours_b,
			    ptrptr_workbench_bw_colours_r,
			    ptrptr_workbench_bw_colours_g,
			    ptrptr_workbench_bw_colours_b,
				
                problem_size_row,/* input */
                problem_size_col,
                problem_x_start,
                problem_x_stop,
                problem_y_start,
                problem_y_stop);

    if(ret_val !=0)
    {/* problem with memory allocation */

        printf("problem with problem creation --> ret_val == %i\n", ret_val);
        return -1;

    }/* problem with memory allocation */
    else
    {/* memory allocated */

        /*define problem */

        printf("memory allocated\n");
        
		
		
	    /* set the whole mesh to vacuum */
	
	    i_col_start   = 1;
        i_col_stop    = 401;
	    i_row_start   = 1;
	    i_row_stop    = 401;
        boundary      = 0;
        voltage       = +0.0;

	    lbmn_mesh_xy_D_set_rectangle_ij(
							*ptrptr_V,
							*ptrptr_Bnd,
							problem_size_row,/* size of mesh */
							problem_size_col,
							i_row_start,/* rectangle parameters */
							i_row_stop,
							i_col_start,
							i_col_stop,
							voltage, /*input rectangle values*/
							boundary);
    
        printf("mesh filled with vacuum\n");

        /*======================*
        * set electrode points  *
        *=======================*/
		
		
		/* casing - left */
		
        i_col_start = 1;
	    i_col_stop  = 10;
	    i_row_start = 1;
	    i_row_stop  = 401;
        boundary    = +1;
        voltage     = +0.0;

	    lbmn_mesh_xy_D_set_rectangle_ij(
							*ptrptr_V,
							*ptrptr_Bnd,
							problem_size_row,/* size of mesh */
							problem_size_col,
							i_row_start,/* rectangle parameters */
							i_row_stop,
							i_col_start,
							i_col_stop,
							voltage, /*input rectangle values*/
							boundary);
		
		

		/* casing - right */
		
        i_col_start = 392;
	    i_col_stop  = 401;
	    i_row_start = 1;
	    i_row_stop  = 401;
        boundary    = +1;
        voltage     = +0.0;

	    lbmn_mesh_xy_D_set_rectangle_ij(
							*ptrptr_V,
							*ptrptr_Bnd,
							problem_size_row,/* size of mesh */
							problem_size_col,
							i_row_start,/* rectangle parameters */
							i_row_stop,
							i_col_start,
							i_col_stop,
							voltage, /*input rectangle values*/
							boundary);
	

		/* casing - bottom */
		
        i_col_start = 11;
	    i_col_stop  = 391;
	    i_row_start = 1;
	    i_row_stop  = 10;
        boundary    = +1;
        voltage     = +0.0;

	    lbmn_mesh_xy_D_set_rectangle_ij(
							*ptrptr_V,
							*ptrptr_Bnd,
							problem_size_row,/* size of mesh */
							problem_size_col,
							i_row_start,/* rectangle parameters */
							i_row_stop,
							i_col_start,
							i_col_stop,
							voltage, /*input rectangle values*/
							boundary);

		/* casing - up */
		
        i_col_start = 11;
	    i_col_stop  = 391;
	    i_row_start = 392;
	    i_row_stop  = 401;
        boundary    = +1;
        voltage     = +0.0;

	    lbmn_mesh_xy_D_set_rectangle_ij(
							*ptrptr_V,
							*ptrptr_Bnd,
							problem_size_row,/* size of mesh */
							problem_size_col,
							i_row_start,/* rectangle parameters */
							i_row_stop,
							i_col_start,
							i_col_stop,
							voltage, /*input rectangle values*/
							boundary);



	    /* left plate - vertical */
	
        i_col_start = 101;
	    i_col_stop  = 300;
	    i_row_start = 301;
	    i_row_stop  = 310;
        boundary    = +2;
        voltage     = +1.0;

	    lbmn_mesh_xy_D_set_rectangle_ij(
							*ptrptr_V,
							*ptrptr_Bnd,
							problem_size_row,/* size of mesh */
							problem_size_col,
							i_row_start,/* rectangle parameters */
							i_row_stop,
							i_col_start,
							i_col_stop,
							voltage, /*input rectangle values*/
							boundary);


	    /* right plate B  - vertical */
	
        i_col_start = 101;
	    i_col_stop  = 300;
	    i_row_start = 91;
	    i_row_stop  = 100;
        boundary    = +3;
        voltage     = -1.0;

	    lbmn_mesh_xy_D_set_rectangle_ij(
							*ptrptr_V,
							*ptrptr_Bnd,
							problem_size_row,/* size of mesh */
							problem_size_col,
							i_row_start,/* rectangle parameters */
							i_row_stop,
							i_col_start,
							i_col_stop,
							voltage, /*input rectangle values*/
							boundary);

  	
	
    }/* memory allocated */
    return 0;
}/* lbmn_demo_xy_problem_31_2_plates_401_401_casing */

/*==============================================================*/

int
lbmn_demo_xy_problem_32_2_plates_401_401_casing_v1(
			unsigned char* ptr_is_pre_defined,/*output*/
			unsigned int* ptr_size_row,
            unsigned int* ptr_size_col,
            double* ptr_x_start,
            double* ptr_x_stop,
            double* ptr_y_start,
            double* ptr_y_stop,
			
			double** ptrptr_V,
			unsigned int** ptrptr_Bnd,
			double** ptrptr_E_x,
			double** ptrptr_E_y,
			double** ptrptr_E_1,
			double** ptrptr_E_2,
            unsigned int** ptrptr_isolines_V,
            unsigned int** ptrptr_isolines_E_x,
            unsigned int** ptrptr_isolines_E_y,
            unsigned int** ptrptr_isolines_E_1,
            unsigned int** ptrptr_isolines_E_2,
			unsigned char** ptrptr_workbench_colours_r,
			unsigned char** ptrptr_workbench_colours_g,
			unsigned char** ptrptr_workbench_colours_b,
			unsigned char** ptrptr_workbench_bw_colours_r,
			unsigned char** ptrptr_workbench_bw_colours_g,
			unsigned char** ptrptr_workbench_bw_colours_b)
{
    /* daclaration of variables */

	unsigned int problem_size_row = 401;
    unsigned int problem_size_col = 401;
    double problem_x_start = -200.0;
    double problem_x_stop  = +200.0;
    double problem_y_start = -200.0;
    double problem_y_stop  = +200.0;

    double       voltage        = 0.0;
    unsigned int boundary       = 0;
    unsigned int i_row_start    = 0;
    unsigned int i_row_stop     = 0;
    unsigned int i_col_start    = 0;
    unsigned int i_col_stop     = 0;

	unsigned int i_row_loop = 0;
	unsigned int i_col_loop = 0;

	double x_start = 0.0;
	double x_stop  = 0.0;
	double y_start = 0.0;
	double y_stop  = 0.0;

	double point_x = 0.0;
	double point_y = 0.0;

	double h_x = 0.0;
	double h_y = 0.0;
	
    int ret_val = 0;

    /* instructions */
	
	h_x = (problem_x_stop - problem_x_start)/(double)(problem_size_col -1);
	h_y = (problem_y_stop - problem_y_start)/(double)(problem_size_row -1);

	(*ptr_is_pre_defined) = 0;/* problem to be solved numerically */

    ret_val = lbmn_mesh_xy_problem_allocate(
        	    ptr_size_row,/*output*/
                ptr_size_col,
                ptr_x_start,
                ptr_x_stop,
                ptr_y_start,
                ptr_y_stop,
			    ptrptr_V,
			    ptrptr_Bnd,
			    ptrptr_E_x,
			    ptrptr_E_y,
			    ptrptr_E_1,
			    ptrptr_E_2,
                ptrptr_isolines_V,
                ptrptr_isolines_E_x,
                ptrptr_isolines_E_y,
                ptrptr_isolines_E_1,
                ptrptr_isolines_E_2,
			    ptrptr_workbench_colours_r,
			    ptrptr_workbench_colours_g,
			    ptrptr_workbench_colours_b,
			    ptrptr_workbench_bw_colours_r,
			    ptrptr_workbench_bw_colours_g,
			    ptrptr_workbench_bw_colours_b,
				
                problem_size_row,/* input */
                problem_size_col,
                problem_x_start,
                problem_x_stop,
                problem_y_start,
                problem_y_stop);

    if(ret_val !=0)
    {/* problem with memory allocation */

        printf("problem with problem creation --> ret_val == %i\n", ret_val);
        return -1;

    }/* problem with memory allocation */
    else
    {/* memory allocated */

        /*define problem */

        printf("memory allocated\n");
        
		
		
	    /* set the whole mesh to vacuum */
	
	    i_col_start   = 1;
        i_col_stop    = 401;
	    i_row_start   = 1;
	    i_row_stop    = 401;
        boundary      = 0;
        voltage       = +0.0;

	    lbmn_mesh_xy_D_set_rectangle_ij(
							*ptrptr_V,
							*ptrptr_Bnd,
							problem_size_row,/* size of mesh */
							problem_size_col,
							i_row_start,/* rectangle parameters */
							i_row_stop,
							i_col_start,
							i_col_stop,
							voltage, /*input rectangle values*/
							boundary);
    
        printf("mesh filled with vacuum\n");

        /*======================*
        * set electrode points  *
        *=======================*/
		
		
		/* casing - left */
		
        i_col_start = 1;
	    i_col_stop  = 10;
	    i_row_start = 1;
	    i_row_stop  = 401;
        boundary    = +1;
        voltage     = +0.0;

	    lbmn_mesh_xy_D_set_rectangle_ij(
							*ptrptr_V,
							*ptrptr_Bnd,
							problem_size_row,/* size of mesh */
							problem_size_col,
							i_row_start,/* rectangle parameters */
							i_row_stop,
							i_col_start,
							i_col_stop,
							voltage, /*input rectangle values*/
							boundary);
		
		

		/* casing - right */
		
        i_col_start = 392;
	    i_col_stop  = 401;
	    i_row_start = 1;
	    i_row_stop  = 401;
        boundary    = +1;
        voltage     = +0.0;

	    lbmn_mesh_xy_D_set_rectangle_ij(
							*ptrptr_V,
							*ptrptr_Bnd,
							problem_size_row,/* size of mesh */
							problem_size_col,
							i_row_start,/* rectangle parameters */
							i_row_stop,
							i_col_start,
							i_col_stop,
							voltage, /*input rectangle values*/
							boundary);
	

		/* casing - bottom */
		
        i_col_start = 11;
	    i_col_stop  = 391;
	    i_row_start = 1;
	    i_row_stop  = 10;
        boundary    = +1;
        voltage     = +0.0;

	    lbmn_mesh_xy_D_set_rectangle_ij(
							*ptrptr_V,
							*ptrptr_Bnd,
							problem_size_row,/* size of mesh */
							problem_size_col,
							i_row_start,/* rectangle parameters */
							i_row_stop,
							i_col_start,
							i_col_stop,
							voltage, /*input rectangle values*/
							boundary);

		/* casing - up */
		
        i_col_start = 11;
	    i_col_stop  = 391;
	    i_row_start = 392;
	    i_row_stop  = 401;
        boundary    = +1;
        voltage     = +0.0;

	    lbmn_mesh_xy_D_set_rectangle_ij(
							*ptrptr_V,
							*ptrptr_Bnd,
							problem_size_row,/* size of mesh */
							problem_size_col,
							i_row_start,/* rectangle parameters */
							i_row_stop,
							i_col_start,
							i_col_stop,
							voltage, /*input rectangle values*/
							boundary);


		/*=== upper plate ===*/
		x_start = -100.0;
		x_stop  = +100.0;
		y_start = +40.0;
		y_stop  = +50.0;
        boundary    = +2;
        voltage     = +1.0;
		
		for(i_row_loop = 1; i_row_loop <= problem_size_row; i_row_loop++)
		{
			for(i_col_loop = 1; i_col_loop <= problem_size_col; i_col_loop++)
			{
	
				point_x = problem_x_start + (i_col_loop - 1) * h_x;
				point_y = problem_y_start + (i_row_loop - 1) * h_y;
				
				
				if((point_x >= x_start)
					&&
					(point_x <= x_stop)
					&&
					(point_y >= y_start)
					&&
					(point_y <= y_stop)
				)
				{
					lbmn_mesh_xy_D_set_point_ij(
									*ptrptr_V,
									*ptrptr_Bnd,
									problem_size_row,/* size of mesh */
									problem_size_col,
									i_row_loop,/* point position (1,2, .. , n_row)*/
									i_col_loop,/* 1,2,..,n_col*/
									voltage, /*input point values*/
									boundary);
				}
				
			}/* loop col */
		}/* loop row */
		


		/*=== bottom plate ===*/
		x_start  = -100.0;
		x_stop   = +100.0;
		y_start  = -50.0;
		y_stop   = -40.0;
        boundary = +3;
        voltage  = -1.0;
		
		for(i_row_loop = 1; i_row_loop <= problem_size_row; i_row_loop++)
		{
			for(i_col_loop = 1; i_col_loop <= problem_size_col; i_col_loop++)
			{
	
				point_x = problem_x_start + (i_col_loop - 1) * h_x;
				point_y = problem_y_start + (i_row_loop - 1) * h_y;
				
				
				if((point_x >= x_start)
					&&
					(point_x <= x_stop)
					&&
					(point_y >= y_start)
					&&
					(point_y <= y_stop)
				)
				{
					lbmn_mesh_xy_D_set_point_ij(
									*ptrptr_V,
									*ptrptr_Bnd,
									problem_size_row,/* size of mesh */
									problem_size_col,
									i_row_loop,/* point position (1,2, .. , n_row)*/
									i_col_loop,/* 1,2,..,n_col*/
									voltage, /*input point values*/
									boundary);
				}
				
			}/* loop col */
		}/* loop row */

	    	
    }/* memory allocated */
    return 0;
}/* lbmn_demo_xy_problem_32_2_plates_401_401_casing_v1 */

/*=================================================================*/

int
lbmn_demo_xy_problem_33_plane_triode_401_401(
			unsigned char* ptr_is_pre_defined,/*output*/
			unsigned int* ptr_size_row,
            unsigned int* ptr_size_col,
            double* ptr_x_start,
            double* ptr_x_stop,
            double* ptr_y_start,
            double* ptr_y_stop,
			
			double** ptrptr_V,
			unsigned int** ptrptr_Bnd,
			double** ptrptr_E_x,
			double** ptrptr_E_y,
			double** ptrptr_E_1,
			double** ptrptr_E_2,
            unsigned int** ptrptr_isolines_V,
            unsigned int** ptrptr_isolines_E_x,
            unsigned int** ptrptr_isolines_E_y,
            unsigned int** ptrptr_isolines_E_1,
            unsigned int** ptrptr_isolines_E_2,
			unsigned char** ptrptr_workbench_colours_r,
			unsigned char** ptrptr_workbench_colours_g,
			unsigned char** ptrptr_workbench_colours_b,
			unsigned char** ptrptr_workbench_bw_colours_r,
			unsigned char** ptrptr_workbench_bw_colours_g,
			unsigned char** ptrptr_workbench_bw_colours_b)
{
    /* daclaration of variables */

    unsigned int problem_size_row = 401;
    unsigned int problem_size_col = 401;
    double problem_x_start = -200.0;
    double problem_x_stop  = +200.0;
    double problem_y_start = -200.0;
    double problem_y_stop  = +200.0;

    double       voltage        = 0.0;
    unsigned int boundary       = 0;
    unsigned int i_row_start    = 0;
    unsigned int i_row_stop     = 0;
    unsigned int i_col_start    = 0;
    unsigned int i_col_stop     = 0;

	unsigned int i_row_loop  =  0;
	unsigned int i_col_loop  =  0;
	unsigned int i_mesh 	 =  0;
	
	double       mesh_step   = +0.0; /* distace between electrodes  */
	
	double distance = 0.0;
	double electrode_radius = 0.0;
	
	double h_x = 0.0;
	double h_y = 0.0;

	
	double point_electrode_center_x = 0.0;
	double point_electrode_center_y = 0.0;
	
	double x_start = 0.0;
	double x_stop  = 0.0;
	/*
	double y_start = 0.0;
	double y_stop  = 0.0;
	*/
	
	double point_x;
	double point_y;
	
    int ret_val = 0;


    /* instructions */
	
	*(ptr_is_pre_defined) = 0;/* problem to be solved numerically */
	
	h_x = (problem_x_stop - problem_x_start)/(double)(problem_size_col -1);
	h_y = (problem_y_stop - problem_y_start)/(double)(problem_size_row -1);
	
	
	/* try to allocate memory */
    ret_val = lbmn_mesh_xy_problem_allocate(
        	    ptr_size_row,/*output*/
                ptr_size_col,
                ptr_x_start,
                ptr_x_stop,
                ptr_y_start,
                ptr_y_stop,
			    ptrptr_V,
			    ptrptr_Bnd,
			    ptrptr_E_x,
			    ptrptr_E_y,
			    ptrptr_E_1,
			    ptrptr_E_2,
                ptrptr_isolines_V,
                ptrptr_isolines_E_x,
                ptrptr_isolines_E_y,
                ptrptr_isolines_E_1,
                ptrptr_isolines_E_2,
			    ptrptr_workbench_colours_r,
			    ptrptr_workbench_colours_g,
			    ptrptr_workbench_colours_b,
			    ptrptr_workbench_bw_colours_r,
			    ptrptr_workbench_bw_colours_g,
			    ptrptr_workbench_bw_colours_b,
				
                problem_size_row,/* input */
                problem_size_col,
                problem_x_start,
                problem_x_stop,
                problem_y_start,
                problem_y_stop);

    if(ret_val !=0)
    {/* problem with memory allocation */

        printf("problem with problem creation --> ret_val == %i\n", ret_val);
        return -1;

    }/* problem with memory allocation */
    else
    {/* memory allocated */

        /*define problem */

        printf("memory allocated\n");
        

	
	    /* set the whole mesh to vacuum */
	
	    i_col_start   = 1;
        i_col_stop    = 401;
	    i_row_start   = 1;
	    i_row_stop    = 401;
        boundary      = 0;
        voltage       = +0.0;

	    lbmn_mesh_xy_D_set_rectangle_ij(
							*ptrptr_V,
							*ptrptr_Bnd,
							problem_size_row,/* size of mesh */
							problem_size_col,
							i_row_start,/* rectangle parameters */
							i_row_stop,
							i_col_start,
							i_col_stop,
							voltage, /*input rectangle values*/
							boundary);
    
        printf("mesh filled with vacuum\n");



        /*===  set rectangle left ====*/

        /* x_start  = -70;  */
	    x_stop   = -199.5;
	    /* y_start  = -100.0; */
	    /* y_stop   = +100.0; */
        boundary = +1;
        voltage  = +1.0;


		for(i_row_loop = 1; i_row_loop <= problem_size_row; i_row_loop++)
		{
			for(i_col_loop = 1; i_col_loop <= problem_size_col; i_col_loop++)
			{
	
				point_x = problem_x_start + (i_col_loop - 1) * h_x;
				point_y = problem_y_start + (i_row_loop - 1) * h_y;
				
				
				if(point_x <= x_stop
				)
				{
					lbmn_mesh_xy_D_set_point_ij(
									*ptrptr_V,
									*ptrptr_Bnd,
									problem_size_row,/* size of mesh */
									problem_size_col,
									i_row_loop,/* point position (1,2, .. , n_row)*/
									i_col_loop,/* 1,2,..,n_col*/
									voltage, /*input point values*/
									boundary);
				}
				
			}/* loop col */
		}/* loop row */
		



        /*===  set rectangle right ====*/

        x_start  = +199.5;
	    /* x_stop   = -90; */
	    /* y_start  = -100.0; */
	    /* y_stop   = +100.0; */
        boundary = +3;
        voltage  = -1.0;


		for(i_row_loop = 1; i_row_loop <= problem_size_row; i_row_loop++)
		{
			for(i_col_loop = 1; i_col_loop <= problem_size_col; i_col_loop++)
			{
	
				point_x = problem_x_start + (i_col_loop - 1) * h_x;
				point_y = problem_y_start + (i_row_loop - 1) * h_y;
				
				
				if(point_x >= x_start
				)
				{
					lbmn_mesh_xy_D_set_point_ij(
									*ptrptr_V,
									*ptrptr_Bnd,
									problem_size_row,/* size of mesh */
									problem_size_col,
									i_row_loop,/* point position (1,2, .. , n_row)*/
									i_col_loop,/* 1,2,..,n_col*/
									voltage, /*input point values*/
									boundary);
				}
				
			}/* loop col */
		}/* loop row */


		/*=== set round electrode (grid) ===*/

		electrode_radius = +5.0;
		boundary         = +2;
        voltage          = +0.0;

		mesh_step  = + 30.0;
		
		/* upper wires y >= +0.0 */
		for(i_mesh = 0; i_mesh <= 4; i_mesh++)
		{
			
			point_electrode_center_x = +50.0;
			point_electrode_center_y = (double)i_mesh * mesh_step;
		
			for(i_row_loop = 1; i_row_loop <= problem_size_row; i_row_loop++)
			{
				for(i_col_loop = 1; i_col_loop <= problem_size_col; i_col_loop++)
				{
		
					point_x = problem_x_start + (i_col_loop - 1) * h_x;
					point_y = problem_y_start + (i_row_loop - 1) * h_y;
					
					
					distance = sqrt((point_x - point_electrode_center_x)*(point_x - point_electrode_center_x) +
									(point_y - point_electrode_center_y)*(point_y - point_electrode_center_y));
					
					if(distance <= electrode_radius)
					{
						lbmn_mesh_xy_D_set_point_ij(
										*ptrptr_V,
										*ptrptr_Bnd,
										problem_size_row,/* size of mesh */
										problem_size_col,
										i_row_loop,/* point position (1,2, .. , n_row)*/
										i_col_loop,/* 1,2,..,n_col*/
										voltage, /*input point values*/
										boundary);
					}
					
				}/* loop col */
			}/* loop row */
			
		}/* loop mesh point_electrode_center >= 0.0 */
	
		/* bottom wires y < +0.0 */
		for(i_mesh = 1; i_mesh <= 4; i_mesh++)
		{
			
			point_electrode_center_x = +50.0;
			point_electrode_center_y = - (double)i_mesh * mesh_step;
		
			for(i_row_loop = 1; i_row_loop <= problem_size_row; i_row_loop++)
			{
				for(i_col_loop = 1; i_col_loop <= problem_size_col; i_col_loop++)
				{
		
					point_x = problem_x_start + (i_col_loop - 1) * h_x;
					point_y = problem_y_start + (i_row_loop - 1) * h_y;
					
					
					distance = sqrt((point_x - point_electrode_center_x)*(point_x - point_electrode_center_x) +
									(point_y - point_electrode_center_y)*(point_y - point_electrode_center_y));
					
					if(distance <= electrode_radius)
					{
						lbmn_mesh_xy_D_set_point_ij(
										*ptrptr_V,
										*ptrptr_Bnd,
										problem_size_row,/* size of mesh */
										problem_size_col,
										i_row_loop,/* point position (1,2, .. , n_row)*/
										i_col_loop,/* 1,2,..,n_col*/
										voltage, /*input point values*/
										boundary);
					}
					
				}/* loop col */
			}/* loop row */
			
		}/* loop mesh point_electrode_center >= 0.0 */
		
		
		
		
    }/* memory allocated */
    return 0;
	
}/* lbmn_demo_xy_problem_33_plane_triode_401_401 */

/* ==== end of file demo_xy_problems_3.c ==== */
