#include <stdio.h>
#include <stdlib.h>
#include <stdint.h>
#include <string.h>


/*============================================*/

unsigned char tab_PBM_r[10000];
unsigned char tab_PBM_g[10000];
unsigned char tab_PBM_b[10000];

/*=============================================*/

/* PROTOTYPES */

void
fill_tab_PBM(void);

/*============================================*/

void
fill_tab_PBM(void)
{
    unsigned int size_col = 100;
    unsigned int i_row = 0;
    unsigned int i_col = 0;

    /* PBM image is:
    *
    *   grey  | red
    *   ------+------
    *   green | blue
    */

    /* green */
    for(i_row = 1; i_row <= 50; i_row++)
    {
        for(i_col = 1; i_col <= 50; i_col++)
        {
            tab_PBM_r[(i_row-1) * size_col + i_col - 1] = 0;
            tab_PBM_g[(i_row-1) * size_col + i_col - 1] = 255;
            tab_PBM_b[(i_row-1) * size_col + i_col - 1] = 0;
        }
    }


    /* blue */
    for(i_row = 1; i_row <= 50; i_row++)
    {
        for(i_col = 51; i_col <= 100; i_col++)
        {
            tab_PBM_r[(i_row-1) * size_col + i_col - 1] = 0;
            tab_PBM_g[(i_row-1) * size_col + i_col - 1] = 0;
            tab_PBM_b[(i_row-1) * size_col + i_col - 1] = 255;
        }
    }


    /* grey */
    for(i_row = 51; i_row <= 100; i_row++)
    {
        for(i_col = 1; i_col <= 50; i_col++)
        {
            tab_PBM_r[(i_row-1) * size_col + i_col - 1] = 128;
            tab_PBM_g[(i_row-1) * size_col + i_col - 1] = 128;
            tab_PBM_b[(i_row-1) * size_col + i_col - 1] = 128;
        }
    }

    /* red */
    for(i_row = 51; i_row <= 100; i_row++)
    {
        for(i_col = 51; i_col <= 100; i_col++)
        {
            tab_PBM_r[(i_row-1) * size_col + i_col - 1] = 255;
            tab_PBM_g[(i_row-1) * size_col + i_col - 1] = 0;
            tab_PBM_b[(i_row-1) * size_col + i_col - 1] = 0;
        }
    }

    return;

}/* fill_tab_PBM */

/*============================================*/

int
main(void)
{
    FILE *ptr_file_out = NULL;

    uint8_t * ptr_buffer = NULL;

    unsigned int row = 0;
    uint64_t row_long_int = 0;
    
    unsigned int col = 0;
    
    /* width, height, and bitcount are the key factors: */

    uint32_t width  = 100;
    uint32_t height = 100;
    
    uint16_t bitcount = 24;/*<- 24-bit bitmap */

    /* take padding in to account */
    uint32_t width_in_bytes = ((width * bitcount + 31) / 32) * 4;

    /* total image size in bytes, not including header */
    uint32_t imagesize = width_in_bytes * height;

    /* this value is always 40, it's the sizeof(BITMAPINFOHEADER) */
    const uint32_t biSize = 40;

    /* bitmap bits start after headerfile, 
     * this is sizeof(BITMAPFILEHEADER) + sizeof(BITMAPINFOHEADER)
     */
    const uint32_t bfOffBits = 54; 

    /* total file size: */
    uint32_t filesize = 54 + imagesize;

    /* number of planes is usually 1 */
    const uint16_t biPlanes = 1;

    /* create header:
     * copy to buffer instead of BITMAPFILEHEADER and BITMAPINFOHEADER
     * to avoid problems with structure packing
    */
    unsigned char header[54] = { 0 };
	
	
	
	
    memcpy(header, "BM", 2);
    memcpy(header + 2 , &filesize, 4);
    memcpy(header + 10, &bfOffBits, 4);
    memcpy(header + 14, &biSize, 4);
    memcpy(header + 18, &width, 4);
    memcpy(header + 22, &height, 4);
    memcpy(header + 26, &biPlanes, 2);
    memcpy(header + 28, &bitcount, 2);
    memcpy(header + 34, &imagesize, 4);


    fill_tab_PBM();

    /* prepare pixel data: */
    ptr_buffer = malloc(imagesize);

    if(ptr_buffer == NULL)
    {/* malloc error */

        /* ERROR */
        return -1;    
        
    }/* malloc error */
    else
    {/* ptr_buffer allocated sucessfully */

        /* main loop preludium */
        row_long_int = height - 1;

        /* loop: row down to zero */
        while(1)
        {
            row = (uint32_t)row_long_int;

            /* loop: col */
            for(col = 0; col < width; col++)
            {
                /* blue */
                ptr_buffer[row * width_in_bytes + col * 3 + 0] = tab_PBM_b[row * 100 + col];/* width == 100 */

                /* green */
                ptr_buffer[row * width_in_bytes + col * 3 + 1] = tab_PBM_g[row * 100 + col];

                /* red */
                ptr_buffer[row * width_in_bytes + col * 3 + 2] = tab_PBM_r[row * 100 + col];

            }/* loop col */


            /* exit main loop if row == 0 */
            if(row_long_int == 0)
            {
                break;
            }

            /* decrease index */
            row_long_int-- ;

        }/* loop row */
        /* after main loop:
         * buf should have been prepared (blue)
         */
        
        ptr_file_out = fopen("test.bmp", "wb");

        if(ptr_file_out == NULL)
        {/* bmp fopen error */

            free(ptr_buffer);

            return -2;

        }/* bmp fopen error */
        else
        {/* bmp fopen ok */

            fwrite(header, 1, 54, ptr_file_out);

            if(ferror(ptr_file_out))
            {
                free(ptr_buffer);

                return -3;
            }
            /* else OK */
            
            fwrite((char*)ptr_buffer, 1, imagesize, ptr_file_out);

            if(ferror(ptr_file_out))
            {
                free(ptr_buffer);

                return -4;
            }
            /* else OK */

            fclose(ptr_file_out);

        }/* bmp fopen ok*/

        free(ptr_buffer);

    }/* ptr_buffer allocated sucessfully */

    return 0;

}/* main */
